#
# TempurityScanList Utility - NTMS Listing to File Script
# October 2013
# Networked Robotics Corporation 2013
#
# 
# 



import socket
import time
import select
import struct
import sys
import subprocess
import telnetlib
import os
import zlib

#
# Set these variables to the defaults you want for the ntms
# type='1' means probe
#
port1default=8088
port2default=8089
port3default=8090
port4default=8091
typedefault='1'
ipdefault = '192','168','1','44'
maskdefault = '255','255','255','0'
iptemp = '192','168','1','150'
ipstring = '192.168.1.150'
firmware_file = 'ntms_rel16b.nrf'


def unsigned32(n):
    return n & 4294967295

def dump_record(response):
    addr = getlong(response[3:5])
    rec = ":" + addr + ":"
    for i in range(5,69,2):
       rec = rec + getlong(response[i:i+2])
    #checksum = "%08X" % unsigned32(zlib.crc32(response[3:69]))
    checksum = ''
    rec = rec + ":" + checksum
    #print rec
    return rec

def getlong(bstring):
    long = ''
    for intval in struct.unpack('BB', bstring):
        if intval > 15:
            replacestr = '0x'
        else:
            replacestr = 'x'
        long = ''.join([long, hex(intval).replace(replacestr, '')])
    return long.upper()

def get_macaddress(bstring):
    macaddr = ''
    for intval in struct.unpack('BBBBBB', bstring):
        if intval > 15:
            replacestr = '0x'
        else:
            replacestr = 'x'
        macaddr = ''.join([macaddr, hex(intval).replace(replacestr, '')])
    return macaddr.upper()

def get_ipaddress(bstring):
    ipaddr = ''
    for intval in struct.unpack('BBBB',bstring):
        ipaddr = ''.join([ipaddr, str(intval)])
        ipaddr = ''.join([ipaddr, '.'])
    return ipaddr.rstrip('.')

def pingit(ip):
   pingaling = os.popen("ping -n 1 "+ip,"r")
   sys.stdout.flush()
   while (1):
      line = pingaling.readline()
      if line:
         if (line.find('timed out') != -1):
            #print line
            return 0
         elif (line.find('Reply from') != -1):
            #print '!',
            return 1

def verify_software(fwname):
   #create the socket
   sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
   sock.connect((ipstring, 3000))

   ver = getversion(sock, 0)
   print "NTMS reports it's running version " + ver + "\n"

   



def config_ntms():
   ip = struct.pack('BBBB', int(NTMXip[0]), int(NTMXip[1]), int(NTMXip[2]), int(NTMXip[3]))
   mask = struct.pack('BBBB', int(NTMXmask[0]), int(NTMXmask[1]), int(NTMXmask[2]), int(NTMXmask[3]))
   ports = struct.pack('HHHH',p1, p2, p3, p4)
   pkt = "NRS" +ip+mask+item['emac']+d1+d2+d3+d4+ports[1]+ports[0]+ports[3]+ports[2]+ports[5]+ports[4]+ports[7]+ports[6]

   sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
   sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

   ack = 0
   tstart = time.time()
   while ((time.time() < (tstart + 5)) & (ack == 0)):
       print "Sending config packet."
       sock.sendto(pkt, ('<broadcast>', 3000))
       is_readable = [sock]
       is_writable = []
       is_error = []
       r, w, e = select.select(is_readable, is_writable, is_error, 1.0)
       if r:
         channel, info = sock.recvfrom(65535)
         if (debug): print info
         if (item['emac'] == channel[13:19]):
            print "Ack received from " + get_macaddress(channel[13:19])
            tstart = time.time() - 30
            ack = 1
       time.sleep(0.200)
   sock.close()
   if (ack == 0): 
	print "\nERROR: No ack to config packet received...\n"
        print "TEST CASE #3 (IP ADDRESS): FAILED"
        sys.exit()
   return 0

def discover_ntms():
   sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
   sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
   sock.bind(('',3001))
   Responses = []
   debug = False
   tstart = time.time()
   # change the 2 to a higher number to wait longer if you get variable numbers of NTMS units returned
   while (time.time() < (tstart + 2)):
       sock.sendto("NRD", ('<broadcast>', 3000))
       is_readable = [sock]
       is_writable = []
       is_error = []
       tstart2 = time.time()
       while (time.time() < (tstart2 + 0.150)):
          r, w, e = select.select(is_readable, is_writable, is_error, 1.0)
          if r:
            channel, info = sock.recvfrom(65535)
            #print info
            maj_ver = struct.unpack('B',channel[3:4])[0]
            min_ver = struct.unpack('B',channel[4:5])[0]
            ver = str(maj_ver) + "." + str(min_ver)
            #ver = struct.unpack('B',channel[3:4])+"."+struct.unpack('B',channel[4:5])
	    #print ver
            ip=get_ipaddress(channel[5:9])
            mask=get_ipaddress(channel[9:13])
            mac=get_macaddress(channel[13:19])
            encodedmac = channel[13:19]
            found=False
	    for item in Responses:
	       if (item['mac'] == mac): found=True
	    if (found == False):
	       Responses.append({'mac':mac, 'emac':encodedmac, 'ip':ip, 'mask':mask, 'dev':channel[19:23], 'ver':ver})
               print "Found: " + mac
   sock.close()
   return Responses

def getdevice(devicenu):
      if (devicenu == "01"):
            return "NR TPL3 Temperature"
      if (devicenu == "02"):   
            return  "Thermo Revco Freezer"
      if (devicenu == "03"):   
            return  "Thermo Forma Freezer"
      if (devicenu == "04"):   
            return "NR ACL Power"
      if (devicenu == "05"):   
           return  "NR HPL1 Humidity"
      if (devicenu == "06"):   
           return  "NR Power Interface"
      if (devicenu == "07"):   
           return  "NR DCP/RTD/LVP"
      if (devicenu == "08"):   
           return  "Thermo Cryoplus Ln2"      
      if (devicenu == "09"):
           return  "Thermo Forma Incubator"
      else:
           return "None"


def sendandwait(sock, data, quiet):
    for x in range(0,8):
       #print "Sending..."
       sock.send(data)
       is_readable = [sock]
       is_writable = []
       is_error = []
       r, w, e = select.select(is_readable, is_writable, is_error, 2.0)
       if r:
         response, info = sock.recvfrom(8192)   
         #print "Received response..."
         return response
    print "ERROR: No response"
    #sys.exit()

def getversion(sock, quiet):
   response=sendandwait(sock, "NRD", 0)
   print "length " + str(len(response))
   maj_ver = struct.unpack('B',response[3:4])[0]
   min_ver = struct.unpack('B',response[4:5])[0]
   ver = str(maj_ver) + "." + str(min_ver)
   return ver


if __name__ == '__main__':

   debug = False

   print "\n================================================="
   print " Tempurityscanlist - NTMS Device Listing to File  10/07/13\n"


   print "Using discovery protocol to locate NTMS."
   Responses = discover_ntms()
   print "Discovery complete...\n"

   print "NTMS units have been discovered:"
   x = 0
   
   fileHandle = open('TempurityDeviceList.csv','w')
   fileHandle.write(  "Unique ID," +  "IP Address," + "NetMask,"+ "Version,"+ "Port1,"+ "Port2,"+ "Port3,"+ "Port4")
   fileHandle.write("\n") 
   for item in Responses:
      x = x + 1
      
      print str(x) + ". MAC:" + item['mac'],
      print " IP:" + item['ip'],
      print " MASK:" + item['mask'],
      print " Firmware:" + item['ver'],

      macstring = item['mac']
      macstring = macstring.strip() [8:12]
      
      fileHandle.write(  macstring + ", " + item['ip'] + ", " + item['mask'] + ", " + item['ver'] + ", ")
      
      arpclear = os.popen("arp -d *") 
      ipstring = item['ip']
     
      try:
         tn=telnetlib.Telnet(ipstring,"3000")
      except:
         print "ERROR: no connection to port 3000"
      
    
      rsp=tn.read_until("xx",1)
      tn.write("c")
      time.sleep(1)
      
      version = tn.read_until("xx",1).strip() [12:23]
      print version

      read1 = version
      read2 = version
      read3 = version
      read4 = version

      read1=read1.strip() [0:2]  
      read2=read2.strip() [3:5] 
      read3=read3.strip() [6:8] 
      read4=read4.strip() [9:11]
           
      devicename = getdevice(read1)
      print devicename,
      fileHandle.write( devicename + ", ")
     
      devicename = getdevice(read2)
      print devicename,
      fileHandle.write( devicename + ", ")
      
      devicename = getdevice(read3)
      print devicename,
      fileHandle.write( devicename + ", ")
      
      devicename = getdevice(read4)
      print devicename
      fileHandle.write( devicename + ", ")
      fileHandle.write("\n")    
      tn.close()

   print "Number of NTMS units discovered: ", x
   #for item in Responses:
   fileHandle.close()
   item = Responses[0]
   macaddress = item['mac']
  

   
   
   

   

   
   #time.sleep(5)



   
   #t1a = tn.read_until("xx",1).strip()[0:6]
    

   

 
  


   

   r = os.popen("notepad TempurityDeviceList.csv")
    


